-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

-- Prep the module
local M = {}
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 11000

local ugfxCalled = 0

-- For launch cycle progress
local launchDuration = 0.2
local launchElapsed = 0
local triggerStale = 0
local launch4Ever = 0

-- Mortar Firing Direction
-- 0 = Not firing
-- 1 = Firing forward
-- 2 = Firing reverse

local function timedLaunch(mortarDir)
	
	-- Trigger is not stale
	if triggerStale == 0 then
		if mortarDir == 1 then
			electrics.values['mortarFiring'] = 1
		else
			electrics.values['mortarFiringR'] = 1
		end
		launchElapsed = 0
		
		-- Trigger has been processed; Comment next line to enable auto-retriggering
		triggerStale = 1
	end
end

local function timedUpdate(dt)
	
	-- Mortar firing time has elapsed; Shut down thrusters
	launchElapsed = launchElapsed + dt
	if launchElapsed >= launchDuration then
		electrics.values['mortarFiring'] = 0
		electrics.values['mortarFiringR'] = 0
	end
end

local function continuousLaunch(mortarDir)
	if mortarDir == 1 then
		electrics.values['mortarFiring'] = 1
	else
		electrics.values['mortarFiringR'] = 1
	end
end

local function continuousUpdate()
	electrics.values['mortarFiring'] = 0
	electrics.values['mortarFiringR'] = 0
end

-- Graphics update function; Updates other stuff too according to dt
local function updateGFX(dt)
	
	-- Get some current elecrtics states
	local mortarTrigger = (electrics.values['mortarTrigger'] or 0)
	local mortarTriggerR = (electrics.values['mortarTriggerR'] or 0)
	local mortarFiring = (electrics.values['mortarFiring'] or 0)
	local mortarFiringR = (electrics.values['mortarFiringR'] or 0)
	local mortarDir
	
	-- Mortar not firing
	if mortarFiring == 0 and mortarFiringR == 0 then
		
		-- Stunt mortar has been commanded to activate
		if mortarTrigger == 1 or mortarTriggerR == 1 then
			if mortarTrigger == 1 then
				mortarDir = 1
			else
				mortarDir = 2
			end
			if launch4Ever > 0.5 then
				continuousLaunch(mortarDir)
			else
				timedLaunch(mortarDir)
			end
			
		-- Trigger is stale until released
		else
			triggerStale = 0
		end
		
	-- Mortar firing or already fired
	else
		if launch4Ever > 0.5 then
			if mortarTrigger == 0 then
				continuousUpdate()
			end
		else
			timedUpdate(dt)
		end
	end
	
	-- Debug message
	if ugfxCalled == 0 then
		ugfxCalled = 1
		print("Stunt Mortar updateGFX called")
	end
end

-- Reset what we need to
local function reset()
	print("Stunt Mortar reset called")
end

-- Initialize everything
local function init(jbeamdata)
	launchDuration = (v.data.variables["$hb9k_forceDur"].val or 200) / 1000
	launch4Ever = (v.data.variables["$hb9k_force4Ever"].val or 0)
	print("Stunt Mortar init called")
end

-- Public interface
M.init = init
M.reset = reset
M.updateGFX = updateGFX

-- Return the module
return M
